#include "GLWindow.h"
#include <iostream>
#include "ngl/Camera.h"
#include "ngl/Colour.h"
#include "ngl/Light.h"
#include "ngl/Matrix.h"
#include "ngl/Transformation.h"
#include "ngl/TransformStack.h"
#include "ngl/Material.h"
#include "ngl/NGLInit.h"
#include "ngl/Obj.h"
#include "ngl/VBOPrimitives.h"
#include "ngl/ShaderManager.h"
#include "ngl/Util.h"
#include "ngl/Random.h"
#include <boost/foreach.hpp>
#include "ngl/Util.h"
#include <ctime>


//----------------------------------------------------------------------------------------------------------------------
GLWindow::GLWindow(
                   QWidget *_parent
                  ) :
                    QGLWidget(_parent)
{

  // set this widget to have the initial keyboard focus
  setFocus();
  // re-size the widget to that of the parent (in this case the GLFrame passed in on construction)
  this->resize(_parent->size());
  // Now set the initial GLWindow attributes to default values
  // Roate is false
  m_rotate=false;
  // mouse rotation values set to 0
  m_spinXFace=0;
  m_spinYFace=0;
  m_origX=0;
  m_origY=0;
  m_lightChangeTimer=startTimer(100);
  m_lightParamChangeTimer=startTimer(1200);
  m_animate=true;
}

GLWindow::~GLWindow()
{
  ngl::NGLInit *Init = ngl::NGLInit::instance();
  std::cout<<"Shutting down NGL, removing VBO's and Shaders\n";
  Init->NGLQuit();
}

// This virtual function is called once before the first call to paintGL() or resizeGL(),
//and then once whenever the widget has been assigned a new QGLContext.
// This function should set up any required OpenGL context rendering flags, defining display lists, etc.

//----------------------------------------------------------------------------------------------------------------------
void GLWindow::initializeGL()
{
  // create the lights
  createLights();

  glClearColor(0.4f, 0.4f, 0.4f, 1.0f);			   // Grey Background
  // enable depth testing for drawing
   glEnable(GL_DEPTH_TEST);
   // Now we will create a basic Camera from the graphics library
   // This is a static camera so it only needs to be set once
   // First create Values for the camera position
   ngl::Vector From(10,4,2);
   ngl::Vector To(0,0,0);
   ngl::Vector Up(0,1,0);
   ngl::NGLInit *Init = ngl::NGLInit::instance();
   #ifdef WIN32
   	glewInit();
   #endif
   Init->initGlew();
   m_cam= new ngl::Camera(From,To,Up,ngl::PERSPECTIVE);
   // set the shape using FOV 45 Aspect Ratio based on Width and Height
   // The final two are near and far clipping planes of 0.5 and 10
   m_cam->setShape(45,(float)720.0/576.0,0.5,150,ngl::PERSPECTIVE);
   // now to load the shader and set the values
   // grab an instance of shader manager
   ngl::ShaderManager *shader=ngl::ShaderManager::instance();
   // load a frag and vert shaders
   shader->loadShader("gl3xTest","shaders/Vertex.vs","shaders/Fragment.fs");
   // set this as the active shader
   shader->useShader("gl3xTest");
   // now pass the modelView and projection values to the shader
   shader->setShaderParamFromMatrix("gl3xTest","ViewMatrix",m_cam->getModelView());
   shader->setShaderParamFromMatrix("gl3xTest","projectionMatrix",m_cam->getProjection());
   shader->setShaderParam1i("gl3xTest","Normalize",true);
   shader->setShaderParam1i("gl3xTest","numLightsEnabled",8);
   shader->setShaderParam4f("gl3xTest","eye",10,4,2,1);

   glEnable(GL_DEPTH_TEST); // for removal of hidden surfaces
   //glEnable(GL_NORMALIZE); // normalize vectors for proper shading
   ngl::Material m(ngl::POLISHEDSILVER);
   m.use();
   ngl::VBOPrimitives *prim =  ngl::VBOPrimitives::instance();
   prim->createVBOQuadPlane("plane",30,30,180,180,ngl::Vector(0,1,0));


}

//----------------------------------------------------------------------------------------------------------------------
//This virtual function is called whenever the widget has been resized.
// The new size is passed in width and height.
void GLWindow::resizeGL(
                        int _w,
                        int _h
                       )
{
  glViewport(0,0,_w,_h);
  m_cam->setShape(45,(float)_w/_h,0.5,150,ngl::PERSPECTIVE);
  ngl::ShaderManager *shader=ngl::ShaderManager::instance();
  shader->setShaderParamFromMatrix("gl3xTest","projectionMatrix",m_cam->getProjection());
}

//----------------------------------------------------------------------------------------------------------------------
//This virtual function is called whenever the widget needs to be painted.
// this is our main drawing routine
void GLWindow::paintGL()
{
  // clear the screen and depth buffer
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 // build our transform stack
  ngl::Transformation trans;
  // set the mouse rotation
  trans.setRotation(m_spinXFace,m_spinYFace,0);
  // set this in the TX stack
  m_transformStack.setGlobal(trans);

  // grab an instance of the primitives for drawing
  ngl::VBOPrimitives *prim=ngl::VBOPrimitives::instance();
  // Rotation based on the mouse position for our global
  // transform
  // grab an instance of the shader manager

  ngl::ShaderManager *shader=ngl::ShaderManager::instance();
  m_transformStack.loadGlobalAndCurrentMatrixToShader("gl3xTest","ModelMatrix");
  shader->setShaderParamFromMatrix("gl3xTest","ModelMatrix",m_transformStack.getCurrAndGlobal().getMatrix());
  prim->draw("plane");

  for (int z=-4; z<5; z+=2)
  {
  for (int x=-4; x<5; x+=2)
  {
  m_transformStack.pushTransform();
  {
    m_transformStack.getCurrentTransform().setRotation(0,(x*z)*20,0);

     m_transformStack.getCurrentTransform().setPosition(ngl::Vector(x,0.49,z));
     shader->setShaderParamFromMatrix("gl3xTest","ModelMatrix",m_transformStack.getCurrAndGlobal().getMatrix());

     prim->draw("teapot");
   } // and before a pop
   m_transformStack.popTransform();

 }
}
}

//----------------------------------------------------------------------------------------------------------------------
void GLWindow::mouseMoveEvent (
                               QMouseEvent * _event
                              )
{
  // note the method buttons() is the button state when event was called
  // this is different from button() which is used to check which button was
  // pressed when the mousePress/Release event is generated
  if(m_rotate && _event->buttons() == Qt::LeftButton)
  {
    m_spinYFace = ( m_spinYFace + (_event->x() - m_origX) ) % 360 ;
    m_spinXFace = ( m_spinXFace + (_event->y() - m_origY) ) % 360 ;
    m_origX = _event->x();
    m_origY = _event->y();
  }
  // re-draw GL
  updateGL();
}


//----------------------------------------------------------------------------------------------------------------------
void GLWindow::mousePressEvent (
                                QMouseEvent * _event
                               )
{
  // this method is called when the mouse button is pressed in this case we
  // store the value where the maouse was clicked (x,y) and set the Rotate flag to true
  if(_event->button() == Qt::LeftButton)
  {
    m_origX = _event->x();
    m_origY = _event->y();
    m_rotate =true;
  }
}

//----------------------------------------------------------------------------------------------------------------------
void GLWindow::mouseReleaseEvent (
                                  QMouseEvent * _event
                                 )
{
  // this event is called when the mouse button is released
  // we then set Rotate to false
  if (_event->button() == Qt::LeftButton)
  {
    m_rotate=false;
  }
}

void GLWindow::createLights()
{
  ngl::Real x=0;
  ngl::Real z=0;
  // loop an set the spot values
  ngl::Random *rand=ngl::Random::instance();
  rand->setSeed(12345);
  for(int i=0; i<8; ++i)
  {
    x=rand->randomNumber(4);
    z=rand->randomNumber(4);
    ngl::Vector pos(x,4,z,1);
    ngl::Vector dir(0,-1,0,0);
    ngl::Colour c=rand->getRandomColour();
    c.clamp(0.5,1.0);

    SpotData d;
    d.m_aimCenter=rand->getRandomPoint(4,0,14);
    d.m_radiusX=rand->randomNumber(1)+0.5;
    d.m_radiusZ=rand->randomNumber(1)+0.5;
    d.m_startColour =rand->getRandomColour();
    d.m_endColour =rand->getRandomColour();
    d.m_startColour.clamp(0.2,1.0);
    d.m_endColour.clamp(0.2,1.0);

    d.m_mix=0.0;

    d.m_time=rand->randomPositiveNumber(3)+0.6;
    m_spotData.push_back(d);
    m_spots.push_back(ngl::SpotLight(pos,dir,c));

  }
  BOOST_FOREACH(ngl::SpotLight &s, m_spots)
  {
    s.setSpecColour(ngl::Colour(1,1,1,1));
    s.setParams(rand->randomPositiveNumber(10)+1,rand->randomPositiveNumber(2)+1,0.4,0,0);
    s.enable();
  }
}
void GLWindow::changeSpotParams()
{
  ngl::Random *rand=ngl::Random::instance();
  rand->setSeed(time(NULL));
  for(int i=0; i<8; ++i)
  {
    float x=rand->randomNumber(4);
    float z=rand->randomNumber(4);
    m_spots[i].setPosition(ngl::Vector(x,4,z,1));

  }
  int size=m_spotData.size();
  for(int i=0; i<size; ++i)
  {
    m_spotData[i].m_aimCenter=rand->getRandomPoint(4,0,14);
    m_spotData[i].m_radiusX=rand->randomNumber(2)+0.5;
    m_spotData[i].m_radiusZ=rand->randomNumber(2)+0.5;
    m_spotData[i].m_time=rand->randomPositiveNumber(4)+0.6;
    m_spotData[i].m_startColour =rand->getRandomColour();
    m_spotData[i].m_endColour =rand->getRandomColour();
    m_spotData[i].m_mix=0.0;
  }
}
void GLWindow::timerEvent(
                          QTimerEvent *_event
                         )
{
  if(_event->timerId() == m_lightChangeTimer)
  {
   if (m_animate !=true)
   {
    return;
   }
   static float time=0.0;
   // create pointers to the begining and end of the spot vector
    int size=m_spots.size();
    for(int i=0; i<size; ++i)
    {
      float pointOnCircleX= cos(time+m_spotData[i].m_time)*m_spotData[i].m_radiusX;
      float pointOnCircleZ= sin(time+m_spotData[i].m_time)*m_spotData[i].m_radiusZ;
      ngl::Vector p(m_spotData[i].m_aimCenter[0]+pointOnCircleX,0,m_spotData[i].m_aimCenter[2]+pointOnCircleZ,0.0);
      m_spots[i].setColour(trigInterp(m_spotData[i].m_startColour,m_spotData[i].m_endColour,m_spotData[i].m_mix ));
      m_spotData[i].m_mix+=0.05;
      if (m_spotData[i].m_mix >=1.0)
      {
        m_spotData[i].m_mix=0.0;
      }
      m_spots[i].aim(p);
      m_spots[i].enable();
     }
    time+=0.1;
    updateGL();

  }
}

